import React, { useState, useEffect, useRef } from 'react';
import { utils } from '@ray-js/panel-sdk';
import { View, Image } from '@ray-js/ray';
import styles from './index.module.less';
import { Props } from './index.type';
import imgs from './colorBg.png';

const { hsv2rgbString } = utils;

// 平移法
const dataStates = {
  // 元素偏移
  translateX: 0,
  translateY: 0,

  // 点击时的小球位置
  locationX: 0,
  locationY: 0,
};

const ColourHeader: React.FC<Props> = props => {
  const data = {
    radius: 135, // 容器半径
    innerRadius: 76, // 内半径
    thumbRadius: 27.5, // 圆球半径
    thumbInnerRadius: 22.5, // 圆球内环
    ringSize: 135 - 76, // 圆环尺寸

    colorThumRadius: 76 - 5,

    cx: 135 - 27.5, // 小球在圆心时相对父容器left位置
    cy: 135 - 27.5, // 小球在圆心时相对父容器top位置

    fixedLength: 135 - (135 - 76) * 0.5, // 可拖动圆球至原点的固定距离（令圆球始终在在色环中居中）
  };

  const sectionRingRef = useRef(null);

  const hueToColor = (hue: number, temp = 1000, bright = 1000, a = 1) => {
    const _hue = Math.abs(hue) % 360;
    return hsv2rgbString(_hue, temp / 10, bright / 10, a);
  };

  const getTransLateByHue = (hue: number) => {
    const rad = ((360 - hue) * Math.PI) / 180;
    const x = data.cx + data.fixedLength * Math.cos(rad);
    const y = data.cy + data.fixedLength * Math.sin(rad);
    return { x, y };
  };

  const [translateX, setTranslateX] = useState(getTransLateByHue(props.hue).x);
  const [translateY, setTranslateY] = useState(getTransLateByHue(props.hue).y);
  const [hueState, sethueState] = useState(props.hue);

  const handleBarTouch = (e: any) => {
    e.origin.stopPropagation();
    dataStates.locationX = e.origin.touches[0].clientX;
    dataStates.locationY = e.origin.touches[0].clientY;

    props.onBarTouch && props.onBarTouch();
  };

  const getHueByTransLate = (LX: number, LY: number) => {
    // 0 ~ 2π
    const rad = getRadianByTransLate(LX, LY);
    return (rad * 180) / Math.PI;
  };

  const getRadianByTransLate = (LX: number, LY: number) => {
    const { thumbRadius } = data;
    // 圆心相对中心点的坐标
    const xCenter = LX - data.cx - thumbRadius;
    const yCenter = LY - data.cy - thumbRadius;

    let rad = Math.atan2(yCenter, xCenter);
    if (xCenter > 0 && yCenter > 0) rad = Math.PI * 2 - rad;
    if (xCenter < 0 && yCenter > 0) rad = Math.PI * 2 - rad;
    if (xCenter < 0 && yCenter < 0) rad = Math.abs(rad);
    if (xCenter > 0 && yCenter < 0) rad = Math.abs(rad);
    if (xCenter === 0 && yCenter > 0) rad = (Math.PI * 3) / 2;
    if (xCenter === 0 && yCenter < 0) rad = Math.PI / 2;
    return rad;
  };

  const handleMove = (e: any) => {
    e.origin.stopPropagation();
    // 累计移动路程，LX的正负决定了移动的正负
    const { locationX, locationY } = dataStates;
    const LX = e.origin.touches[0].clientX - locationX + dataStates.translateX;
    const LY = e.origin.touches[0].clientY - locationY + dataStates.translateY;
    const hue = Math.round(getHueByTransLate(LX, LY));
    const { x = 0, y = 0 } = getTransLateByHue(hue);

    const color = hueToColor(hue);
    setTranslateX(x);
    setTranslateY(y);
    sethueState(hue);

    props.onBarMove && props.onBarMove(hue, color);
  };

  const handleEnd = () => {
    dataStates.translateX = translateX;
    dataStates.translateY = translateY;

    props.onBarMoveEnd && props.onBarMoveEnd(hueState);
  };

  const handleRingClick = e => {
    console.log(e, sectionRingRef);
    // 因为无法获取到指定元素的位置，所以无法实现让小球处于点击位置
  };

  return (
    <View className={styles.Ring}>
      <View
        className={styles.RingContent}
        style={{ width: `${data.radius * 2}px`, height: `${data.radius * 2}px` }}
      >
        {/* 圆环 */}
        <View
          ref={sectionRingRef}
          className={styles.sectionRing}
          style={{ width: `${data.radius * 2}px`, height: `${data.radius * 2}px` }}
          onClick={handleRingClick}
        >
          <Image
            style={{
              width: `${data.radius * 2}px`,
              height: `${data.radius * 2}px`,
              borderRadius: '50%',
            }}
            src={imgs}
          />
        </View>

        {/* 圆环中间色彩值 */}
        <View
          className={styles.colorThum}
          style={{
            position: 'absolute',
            left: `${data.radius - data.colorThumRadius}px`,
            top: `${data.radius - data.colorThumRadius}px`,
            width: `${data.colorThumRadius * 2}px`,
            height: `${data.colorThumRadius * 2}px`,
            borderRadius: '50%',
            backgroundColor: hueToColor(hueState, props.temp, props.bright),
          }}
        >
          Angle:{hueState}°
        </View>

        {/* 圆球 */}
        <View
          className={styles.bar}
          style={{
            width: `${data.thumbRadius * 2}px`,
            height: `${data.thumbRadius * 2}px`,
            borderRadius: '50%',
            opacity: 1,
            transform: `translateX(${translateX}px) translateY(${translateY}px)`,
          }}
          onTouchStart={handleBarTouch}
          onTouchMove={handleMove}
          onTouchEnd={handleEnd}
        >
          <View
            style={{
              width: `${data.thumbInnerRadius * 2}px`,
              height: `${data.thumbInnerRadius * 2}px`,
              borderRadius: '50%',
              backgroundColor: 'transparent',
            }}
          />
        </View>
      </View>
    </View>
  );
};

export default ColourHeader;
